from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class InlineResponse200_100DataRenewResult(BaseModel):
    """InlineResponse200_100DataRenewResult

    :param redirect_to_app_type_page: redirect_to_app_type_page, defaults to None
    :type redirect_to_app_type_page: bool, optional
    :param need_to_renew: need_to_renew, defaults to None
    :type need_to_renew: bool, optional
    """

    def __init__(
        self, redirect_to_app_type_page: bool = None, need_to_renew: bool = None
    ):
        """InlineResponse200_100DataRenewResult

        :param redirect_to_app_type_page: redirect_to_app_type_page, defaults to None
        :type redirect_to_app_type_page: bool, optional
        :param need_to_renew: need_to_renew, defaults to None
        :type need_to_renew: bool, optional
        """
        self.redirect_to_app_type_page = redirect_to_app_type_page
        self.need_to_renew = need_to_renew



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class OrderResponseStatusTransitions(BaseModel):
    """Indicates the last time in Unix time that the order transitioned to one of the following statuses. A zero value for a status indicates that the order has never transitioned to it.

    :param canceled: canceled, defaults to None
    :type canceled: float, optional
    :param fulfilled: fulfilled, defaults to None
    :type fulfilled: float, optional
    :param paid: paid, defaults to None
    :type paid: float, optional
    :param returned: returned, defaults to None
    :type returned: float, optional
    :param pending: pending, defaults to None
    :type pending: float, optional
    :param partial: partial, defaults to None
    :type partial: float, optional
    """

    def __init__(
        self,
        canceled: float = None,
        fulfilled: float = None,
        paid: float = None,
        returned: float = None,
        pending: float = None,
        partial: float = None,
    ):
        """Indicates the last time in Unix time that the order transitioned to one of the following statuses. A zero value for a status indicates that the order has never transitioned to it.

        :param canceled: canceled, defaults to None
        :type canceled: float, optional
        :param fulfilled: fulfilled, defaults to None
        :type fulfilled: float, optional
        :param paid: paid, defaults to None
        :type paid: float, optional
        :param returned: returned, defaults to None
        :type returned: float, optional
        :param pending: pending, defaults to None
        :type pending: float, optional
        :param partial: partial, defaults to None
        :type partial: float, optional
        """
        self.canceled = self._define_number("canceled", canceled, nullable=True)
        self.fulfilled = self._define_number("fulfilled", fulfilled, nullable=True)
        self.paid = self._define_number("paid", paid, nullable=True)
        self.returned = self._define_number("returned", returned, nullable=True)
        self.pending = self._define_number("pending", pending, nullable=True)
        self.partial = self._define_number("partial", partial, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class InlineResponse200_65Data(BaseModel):
    """InlineResponse200_65Data

    :param payout_token: ID of the payout. String starting with **payout_**., defaults to None
    :type payout_token: str, optional
    :param file_id: ID of the uploaded file. UUID., defaults to None
    :type file_id: str, optional
    :param file_name: The name of the document file., defaults to None
    :type file_name: str, optional
    :param file_extension: The uploaded file's extension., defaults to None
    :type file_extension: str, optional
    :param created_at: The date and time when the file was successfully uploaded. Format is **YYYY-MM-DD HH:MM:SS**., defaults to None
    :type created_at: str, optional
    """

    def __init__(
        self,
        payout_token: str = None,
        file_id: str = None,
        file_name: str = None,
        file_extension: str = None,
        created_at: str = None,
    ):
        """InlineResponse200_65Data

        :param payout_token: ID of the payout. String starting with **payout_**., defaults to None
        :type payout_token: str, optional
        :param file_id: ID of the uploaded file. UUID., defaults to None
        :type file_id: str, optional
        :param file_name: The name of the document file., defaults to None
        :type file_name: str, optional
        :param file_extension: The uploaded file's extension., defaults to None
        :type file_extension: str, optional
        :param created_at: The date and time when the file was successfully uploaded. Format is **YYYY-MM-DD HH:MM:SS**., defaults to None
        :type created_at: str, optional
        """
        self.payout_token = self._define_str(
            "payout_token", payout_token, nullable=True
        )
        self.file_id = self._define_str("file_id", file_id, nullable=True)
        self.file_name = self._define_str("file_name", file_name, nullable=True)
        self.file_extension = self._define_str(
            "file_extension", file_extension, nullable=True
        )
        self.created_at = self._define_str("created_at", created_at, nullable=True)



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    Category,
    CustomerIdPaymentMethodsBody,
    CustomerPaymentMethod,
    InlineResponse200_41,
    InlineResponse200_42,
    InlineResponse200_43,
)


class CustomerPaymentMethodService(BaseService):

    @cast_models
    def get_customer_payment_methods(
        self,
        customer_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        category: Category = None,
        starting_after: str = None,
        ending_before: str = None,
        limit: str = None,
        type_: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_41:
        """Retrieve payment methods for a customer

        :param customer_id: customer Id
        :type customer_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param category: category, defaults to None
        :type category: Category, optional
        :param starting_after: The ID of the coupon created before the first coupon you want to retrieve, defaults to None
        :type starting_after: str, optional
        :param ending_before: The ID of the coupon created after the last coupon you want to retrieve, defaults to None
        :type ending_before: str, optional
        :param limit: The maximum number of coupons to return. Range is 1-100. Default is 10, defaults to None
        :type limit: str, optional
        :param type_: The type of payment method to find., defaults to None
        :type type_: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: get customer details By customer Id
        :rtype: InlineResponse200_41
        """

        Validator(str).validate(customer_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(Category).is_optional().validate(category)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(type_)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/customers/{{customerId}}/payment_methods",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("customerId", customer_id)
            .add_query("category", category)
            .add_query("starting_after", starting_after)
            .add_query("ending_before", ending_before)
            .add_query("limit", limit)
            .add_query("type", type_)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_41._unmap(response)

    @cast_models
    def create_customer_payment_method(
        self,
        request_body: CustomerIdPaymentMethodsBody,
        customer_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_42:
        """Add a payment method to a customer profile

        :param request_body: The request body.
        :type request_body: CustomerIdPaymentMethodsBody
        :param customer_id: customer Id
        :type customer_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Payment method was created for this customer successfully
        :rtype: InlineResponse200_42
        """

        Validator(CustomerIdPaymentMethodsBody).validate(request_body)
        Validator(str).validate(customer_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/customers/{{customerId}}/payment_methods",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("customerId", customer_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_42._unmap(response)

    @cast_models
    def get_customer_payment_method(
        self,
        customer_id: str,
        pmt_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_42:
        """Retrieve a payment method for a specific customer

        :param customer_id: customer Id
        :type customer_id: str
        :param pmt_id: Pmt Id
        :type pmt_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: get customer details By customer Id
        :rtype: InlineResponse200_42
        """

        Validator(str).validate(customer_id)
        Validator(str).validate(pmt_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/customers/{{customerId}}/payment_methods/{{pmtId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("customerId", customer_id)
            .add_path("pmtId", pmt_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_42._unmap(response)

    @cast_models
    def update_customer_payment_method(
        self,
        request_body: CustomerPaymentMethod,
        customer_id: str,
        pmt_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_42:
        """Change or modify a payment method that was stored in a customer profile

        :param request_body: The request body.
        :type request_body: CustomerPaymentMethod
        :param customer_id: customer Id
        :type customer_id: str
        :param pmt_id: Pmt Id
        :type pmt_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: customer was updated successfully
        :rtype: InlineResponse200_42
        """

        Validator(CustomerPaymentMethod).validate(request_body)
        Validator(str).validate(customer_id)
        Validator(str).validate(pmt_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/customers/{{customerId}}/payment_methods/{{pmtId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("customerId", customer_id)
            .add_path("pmtId", pmt_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_42._unmap(response)

    @cast_models
    def delete_customer_payment_method(
        self,
        customer_id: str,
        pmt_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_43:
        """This method triggers the Webhook - Payment Method Canceled webhook. This webhook contains more information than the response.

        :param customer_id: customer Id
        :type customer_id: str
        :param pmt_id: Pmt Id
        :type pmt_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: payment method was deleted for this customer
        :rtype: InlineResponse200_43
        """

        Validator(str).validate(customer_id)
        Validator(str).validate(pmt_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/customers/{{customerId}}/payment_methods/{{pmtId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("customerId", customer_id)
            .add_path("pmtId", pmt_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_43._unmap(response)



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    BeneficiaryBeneficiaryIdBody,
    BeneficiaryValidateBody,
    DisburseBeneficiaryBody,
    InlineResponse200_53,
    InlineResponse200_54,
    InlineResponse200_55,
    InlineResponse200_56,
    InlineResponse200_57,
    InlineResponse200_58,
    InlineResponse200_59,
    InlineResponse200_60,
    InlineResponse200_61,
    InlineResponse200_62,
    InlineResponse200_63,
    InlineResponse200_64,
    PayoutsBeneficiaryBody,
    PayoutsExtendedBeneficiaryBody,
    PayoutsPayoutIdBody,
    PayoutsSenderBody,
    V1PayoutsBody,
)


class DisburseService(BaseService):

    @cast_models
    def get_payout_method_types_details(
        self,
        pomt: str,
        beneficiary_country: str,
        beneficiary_entity_type: str,
        payout_amount: float,
        payout_currency: str,
        sender_country: str,
        sender_currency: str,
        sender_entity_type: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_53:
        """Retrieve the fields required to use a payout method type. The fields are returned as an array of objects. The name of each field appears in the name field of each object. Use this information for Create Payout, Create Beneficiary and Create Sender.
         Note: The fields returned by this method are required when you create a payout. If you create a payout with a sender or beneficiary that was created previously, you are responsible for choosing a sender and beneficiary that have all the fields required by the payout method.

        :param pomt: The type of the payout method. Set to the name of a payout method listed in the response to List Payout Method Types. The two-letter prefix must match the beneficiary country code.
        :type pomt: str
        :param beneficiary_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. Uppercase.
        :type beneficiary_country: str
        :param beneficiary_entity_type: Type of entity for the beneficiary. One of the following, **individual** | **company**
        :type beneficiary_entity_type: str
        :param payout_amount: Amount of the payout, in units of the currency that the beneficiary is receiving. Decimal.
        :type payout_amount: float
        :param payout_currency: Currency received by the beneficiary. Three-letter ISO 4217 code. Uppercase.
        :type payout_currency: str
        :param sender_country: Country of the sender. Two-letter ISO 3166-1 ALPHA-2 code. Uppercase.
        :type sender_country: str
        :param sender_currency: Currency that the sender is paying with. Three-letter ISO 4217 code. Uppercase.
        :type sender_currency: str
        :param sender_entity_type: Type of entity for the sender. One of the following: **individual** | **company**
        :type sender_entity_type: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: payout method types details fetched successfully
        :rtype: InlineResponse200_53
        """

        Validator(str).validate(pomt)
        Validator(str).validate(beneficiary_country)
        Validator(str).validate(beneficiary_entity_type)
        Validator(float).validate(payout_amount)
        Validator(str).validate(payout_currency)
        Validator(str).validate(sender_country)
        Validator(str).validate(sender_currency)
        Validator(str).validate(sender_entity_type)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payout_methods/{{pomt}}/required_fields",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("pomt", pomt, explode=True)
            .add_query("beneficiary_country", beneficiary_country)
            .add_query("beneficiary_entity_type", beneficiary_entity_type)
            .add_query("payout_amount", payout_amount)
            .add_query("payout_currency", payout_currency)
            .add_query("sender_country", sender_country)
            .add_query("sender_currency", sender_currency)
            .add_query("sender_entity_type", sender_entity_type)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_53._unmap(response)

    @cast_models
    def list_payouts(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        beneficiary: str = None,
        created_after: str = None,
        created_before: str = None,
        ending_before: str = None,
        ewallet: str = None,
        limit: str = None,
        merchant_reference_id: str = None,
        payout_method_type: str = None,
        sender: str = None,
        sender_country: str = None,
        sender_currency: str = None,
        starting_after: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_54:
        """Retrieve a list of payouts that you created.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param beneficiary: Filters according to the beneficiary ID. String starting with **beneficiary_**., defaults to None
        :type beneficiary: str, optional
        :param created_after: The ID of the payout created before the first payout you want to retrieve. String starting with **payout_**., defaults to None
        :type created_after: str, optional
        :param created_before: The ID of the payout created after the last payout you want to retrieve. String starting with **payout_**., defaults to None
        :type created_before: str, optional
        :param ending_before: The ID of a payout in the list. The list ends with the payout that was created before the payout with this ID. Use this filter to get the previous page of results. String starting with **payout_**. Deprecated., defaults to None
        :type ending_before: str, optional
        :param ewallet: Filters according to the wallet ID. String starting with **ewallet_**., defaults to None
        :type ewallet: str, optional
        :param limit: The maximum number of payouts to return. Range: 1-100. Default is 10., defaults to None
        :type limit: str, optional
        :param merchant_reference_id: Filters according to the merchant reference ID., defaults to None
        :type merchant_reference_id: str, optional
        :param payout_method_type: Filters according to the type of payout method. The two-letter prefix must match the beneficiary country code., defaults to None
        :type payout_method_type: str, optional
        :param sender: Filters according to the sender ID. String starting with **sender_**., defaults to None
        :type sender: str, optional
        :param sender_country: Filters according to the country of the sender. Two-letter ISO 3166-1 ALPHA-2 code., defaults to None
        :type sender_country: str, optional
        :param sender_currency: Filters according to the currency that the sender is paying with. Three-letter ISO 4217 code., defaults to None
        :type sender_currency: str, optional
        :param starting_after: The ID of a payout in the list. The list begins with the payout that was created next after the payout with this ID. Use this filter to get the next page of results. Relevant when ending_before is not used. String starting **payout_**. Deprecated, defaults to None
        :type starting_after: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Payouts Fetched Successfully
        :rtype: InlineResponse200_54
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(beneficiary)
        Validator(str).is_optional().validate(created_after)
        Validator(str).is_optional().validate(created_before)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(ewallet)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(merchant_reference_id)
        Validator(str).is_optional().validate(payout_method_type)
        Validator(str).is_optional().validate(sender)
        Validator(str).is_optional().validate(sender_country)
        Validator(str).is_optional().pattern(
            "/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/"
        ).validate(sender_currency)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/payouts", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("beneficiary", beneficiary)
            .add_query("created_after", created_after)
            .add_query("created_before", created_before)
            .add_query("ending_before", ending_before)
            .add_query("ewallet", ewallet)
            .add_query("limit", limit)
            .add_query("merchant_reference_id", merchant_reference_id)
            .add_query("payout_method_type", payout_method_type)
            .add_query("sender", sender)
            .add_query("sender_country", sender_country)
            .add_query("sender_currency", sender_currency)
            .add_query("starting_after", starting_after)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_54._unmap(response)

    @cast_models
    def create_payout(
        self,
        request_body: V1PayoutsBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_55:
        """Create a payout (disbursement). This method triggers the Payout Created webhook. This webhook contains the same information as the response. If the action of a third party is not required, the Payout Completed webhook is also triggered.

        :param request_body: The request body.
        :type request_body: V1PayoutsBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: payout created successfully
        :rtype: InlineResponse200_55
        """

        Validator(V1PayoutsBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/payouts", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_55._unmap(response)

    @cast_models
    def create_mass_payout(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_56:
        """Create a batch of payouts (disbursements) from a file. To enable the use of this method, contact Rapyd support. A request input file must be added after the header parameters in the following format: <BR> `-F 'file=@/your-file-location/mass-payout-file.csv'`

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Mass payout created successfully
        :rtype: InlineResponse200_56
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/batch_process/files/mass_payout_pci",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_56._unmap(response)

    @cast_models
    def create_beneficiary(
        self,
        request_body: PayoutsBeneficiaryBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_57:
        """Create a beneficiary for use in payouts.
        The response provides a unique beneficiary ID, which you can use in place of the Beneficiary object for Create Payout.
        This method triggers the Beneficiary Created webhook. This webhook contains the same information as the response.
        Note: In addition to the required fields documented below, you must include all other beneficiary fields listed in the response to Get Payout Required Fields, and you must conform to the regex provided.
        To create a beneficiary that you can use with multiple payout methods, include all fields that are required by each payout method.
        The client is responsible for including all required fields.

        :param request_body: The request body.
        :type request_body: PayoutsBeneficiaryBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Beneficiary Created Successfully
        :rtype: InlineResponse200_57
        """

        Validator(PayoutsBeneficiaryBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/beneficiary", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_57._unmap(response)

    @cast_models
    def create_extended_beneficiary(
        self,
        request_body: PayoutsExtendedBeneficiaryBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_57:
        """Create a beneficiary that includes all compliance related data for payouts. The response provides a unique beneficiary ID, which you can use in place of the beneficiary object for Create Payout

        :param request_body: The request body.
        :type request_body: PayoutsExtendedBeneficiaryBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Beneficiary Created Successfully
        :rtype: InlineResponse200_57
        """

        Validator(PayoutsExtendedBeneficiaryBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/extended_beneficiary",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_57._unmap(response)

    @cast_models
    def validate_beneficiary(
        self,
        request_body: BeneficiaryValidateBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_58:
        """Validate the format of the details for a payout beneficiary. You can validate a beneficiary multiple times for different payout methods. You can enter a Beneficiary object or a previously created beneficiary ID.
         Note: In addition to the required fields for Create Beneficiary, the Beneficiary object must include all required fields for the beneficiary listed in the response to Get Payout Required Fields. The examples include additional fields for the 'us_ach_bank' payout method.

        :param request_body: The request body.
        :type request_body: BeneficiaryValidateBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Beneficiary validated successfully
        :rtype: InlineResponse200_58
        """

        Validator(BeneficiaryValidateBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/beneficiary/validate",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_58._unmap(response)

    @cast_models
    def get_beneficiary(
        self,
        beneficiary_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_57:
        """Retrieve details of a payout beneficiary.
         Note: The 'Retrieve Beneficiary - individual' response includes additional fields for the 'ca_general_bank' payout method. The 'Retrieve Beneficiary - company' response includes additional fields for the 'us_ach_bank' payout method.

        :param beneficiary_id: ID of the 'beneficiary' object. String starting with **beneficiary_**.
        :type beneficiary_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Beneficiary retrieved successfully
        :rtype: InlineResponse200_57
        """

        Validator(str).validate(beneficiary_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/beneficiary/{{beneficiaryId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("beneficiaryId", beneficiary_id, explode=True)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_57._unmap(response)

    @cast_models
    def update_beneficiary(
        self,
        request_body: BeneficiaryBeneficiaryIdBody,
        beneficiary_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_57:
        """Update a beneficiary with additional fields. <BR>Note: You cannot update the values of the beneficiary fields which were defined when it was created.

        :param request_body: The request body.
        :type request_body: BeneficiaryBeneficiaryIdBody
        :param beneficiary_id: ID of the 'beneficiary' object. String starting with **beneficiary_**.
        :type beneficiary_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Beneficiary updated successfully
        :rtype: InlineResponse200_57
        """

        Validator(BeneficiaryBeneficiaryIdBody).validate(request_body)
        Validator(str).validate(beneficiary_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/beneficiary/{{beneficiaryId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("beneficiaryId", beneficiary_id, explode=True)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_57._unmap(response)

    @cast_models
    def delete_beneficiary(
        self,
        beneficiary_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_59:
        """Delete a payout beneficiary from the Rapyd platform.

        :param beneficiary_id: ID of the 'beneficiary' object. String starting with **beneficiary_**.
        :type beneficiary_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Beneficiary deleted successfully
        :rtype: InlineResponse200_59
        """

        Validator(str).validate(beneficiary_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/beneficiary/{{beneficiaryId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("beneficiaryId", beneficiary_id, explode=True)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_59._unmap(response)

    @cast_models
    def simulate_complete_payout(
        self,
        payout_token: str,
        amount: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_55:
        """simulate_complete_payout

        :param payout_token: ID of the payout. String starting with **payout_**.
        :type payout_token: str
        :param amount: The payout amount. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015.
        :type amount: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The `Retrieve Sender - individual` response includes additional fields for the `ca_general_bank` payout method. The `Retrieve Sender - company` response includes an additional field for the `us_ach_bank` payout method.
        :rtype: InlineResponse200_55
        """

        Validator(str).validate(payout_token)
        Validator(str).validate(amount)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/complete/{{payoutToken}}/{{amount}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payoutToken", payout_token, explode=True)
            .add_path("amount", amount, explode=True)
            .serialize()
            .set_method("POST")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_55._unmap(response)

    @cast_models
    def beneficiary_tokenization_page(
        self,
        request_body: DisburseBeneficiaryBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_60:
        """Create a customer-facing page for creating a beneficiary token.<BR> This method creates a URL for a page where the customer enters details required for payouts to the beneficiary’s bank account. All available payout methods for the beneficiary’s country are listed.

        :param request_body: The request body.
        :type request_body: DisburseBeneficiaryBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Beneficiary tokenization page created.
        :rtype: InlineResponse200_60
        """

        Validator(DisburseBeneficiaryBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/hosted/disburse/beneficiary",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_60._unmap(response)

    @cast_models
    def confirm_payout(
        self,
        payout_token: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_55:
        """confirm_payout

        :param payout_token: ID of the payout. String starting with **payout_**.
        :type payout_token: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The `Retrieve Sender - individual` response includes additional fields for the `ca_general_bank` payout method. The `Retrieve Sender - company` response includes an additional field for the `us_ach_bank` payout method.
        :rtype: InlineResponse200_55
        """

        Validator(str).validate(payout_token)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/confirm/{{payoutToken}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payoutToken", payout_token, explode=True)
            .serialize()
            .set_method("POST")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_55._unmap(response)

    @cast_models
    def create_sender(
        self,
        request_body: PayoutsSenderBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_61:
        """Create a sender for use in payouts.
        The response provides a unique sender ID, which you can use in place of the sender object for Create Payout.
        Note: In addition to the required fields documented below, you must include all other sender fields listed in the response to Get Payout Required Fields, and you must conform to the regex provided.
        To create a sender that you can use with multiple payout methods, include all fields that are required by each payout method.
        The client is responsible for including all required fields.

        :param request_body: The request body.
        :type request_body: PayoutsSenderBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Sender Created Successfully.
        :rtype: InlineResponse200_61
        """

        Validator(PayoutsSenderBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/payouts/sender", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_61._unmap(response)

    @cast_models
    def get_payer(
        self,
        sender_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_61:
        """Note: The Retrieve Sender - individual response includes additional fields for the **ca_general_bank** payout method. The Retrieve Sender - company response includes an additional field for the **us_ach_bank** payout method.

        :param sender_id: ID of the Sender object. String starting with **sender_**.
        :type sender_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The 'Retrieve Sender - individual' response includes additional fields for the 'ca_general_bank' payout method. The 'Retrieve Sender - company' response includes an additional field for the **us_ach_bank** payout method.
        :rtype: InlineResponse200_61
        """

        Validator(str).validate(sender_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/sender/{{senderId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("senderId", sender_id, explode=True)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_61._unmap(response)

    @cast_models
    def delete_payer(
        self,
        sender_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_62:
        """Delete a payout sender from the Rapyd platform.

        :param sender_id: ID of the `sender` object. String starting with **sender_**.
        :type sender_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Sender deleted successfully
        :rtype: InlineResponse200_62
        """

        Validator(str).validate(sender_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/sender/{{senderId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("senderId", sender_id, explode=True)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_62._unmap(response)

    @cast_models
    def get_payout_method_types(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        sender_entity_type: str = None,
        beneficiary_country: str = None,
        payout_currency: str = None,
        sender_currency: str = None,
        sender_country: str = None,
        beneficiary_entity_type: str = None,
        category: str = None,
        is_cancelable: bool = None,
        is_location_specific: bool = None,
        is_expirable: bool = None,
        starting_after: str = None,
        ending_before: str = None,
        limit: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_63:
        """Retrieve a list of payout method types that you can use when creating a payout. The response contains a list of objects. Each object includes a payout method type and its attributes.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param sender_entity_type: Filters the type of entity for the payer. One of the following: **individual** | **company**, defaults to None
        :type sender_entity_type: str, optional
        :param beneficiary_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code., defaults to None
        :type beneficiary_country: str, optional
        :param payout_currency: Currency received by the beneficiary. Three-letter ISO 4217 code., defaults to None
        :type payout_currency: str, optional
        :param sender_currency: Currency that the sender is paying with. Three-letter ISO 4217 code., defaults to None
        :type sender_currency: str, optional
        :param sender_country: Country of the sender. Two-letter ISO 3166-1 ALPHA-2 code. Note: This field does not appear in the response., defaults to None
        :type sender_country: str, optional
        :param beneficiary_entity_type: Filters the type of entity for the beneficiary. One of the following: **individual** | **company**, defaults to None
        :type beneficiary_entity_type: str, optional
        :param category: The category of payout method. One of the following: bank, card, cash, rapyd_ewallet, ewallet, defaults to None
        :type category: str, optional
        :param is_cancelable: Indicates whether the payout can be canceled. Relevant when category is cash., defaults to None
        :type is_cancelable: bool, optional
        :param is_location_specific: Indicates whether the payout must be made at a specific physical location. Relevant when category is cash., defaults to None
        :type is_location_specific: bool, optional
        :param is_expirable: Indicates whether the payout expires if not completed. Relevant when category is cash., defaults to None
        :type is_expirable: bool, optional
        :param starting_after: The name of a payout method in the list. The list begins with the record that was created next after the record with this payout method. Use this filter to get the next page of results. Relevant when ending_before is not used., defaults to None
        :type starting_after: str, optional
        :param ending_before: The name of a payout method in the list. The list ends with the last record that was created before the record with this payout method. Use this filter to get the previous page of results., defaults to None
        :type ending_before: str, optional
        :param limit: The maximum number of payout methods to return. Range: 1-100. Default is 10., defaults to None
        :type limit: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: payout method types fetched successfully
        :rtype: InlineResponse200_63
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(sender_entity_type)
        Validator(str).is_optional().validate(beneficiary_country)
        Validator(str).is_optional().validate(payout_currency)
        Validator(str).is_optional().validate(sender_currency)
        Validator(str).is_optional().validate(sender_country)
        Validator(str).is_optional().validate(beneficiary_entity_type)
        Validator(str).is_optional().validate(category)
        Validator(bool).is_optional().validate(is_cancelable)
        Validator(bool).is_optional().validate(is_location_specific)
        Validator(bool).is_optional().validate(is_expirable)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/payout_methods", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("sender_entity_type", sender_entity_type)
            .add_query("beneficiary_country", beneficiary_country)
            .add_query("payout_currency", payout_currency)
            .add_query("sender_currency", sender_currency)
            .add_query("sender_country", sender_country)
            .add_query("beneficiary_entity_type", beneficiary_entity_type)
            .add_query("category", category)
            .add_query("is_cancelable", is_cancelable)
            .add_query("is_location_specific", is_location_specific)
            .add_query("is_expirable", is_expirable)
            .add_query("starting_after", starting_after)
            .add_query("ending_before", ending_before)
            .add_query("limit", limit)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_63._unmap(response)

    @cast_models
    def get_payout(
        self,
        payout_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_55:
        """Retrieve the details of a payout.

        :param payout_id: ID of the payout. String starting with **payout_**.
        :type payout_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Payout object
        :rtype: InlineResponse200_55
        """

        Validator(str).validate(payout_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/{{payoutId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payoutId", payout_id, explode=True)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_55._unmap(response)

    @cast_models
    def update_payout(
        self,
        request_body: PayoutsPayoutIdBody,
        payout_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_55:
        """Change or modify a payout. This method triggers the Payout Updated webhook. This webhook contains the same information as the response.

        :param request_body: The request body.
        :type request_body: PayoutsPayoutIdBody
        :param payout_id: ID of the payout. String starting with **payout_**.
        :type payout_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: payout updated successfully
        :rtype: InlineResponse200_55
        """

        Validator(PayoutsPayoutIdBody).validate(request_body)
        Validator(str).validate(payout_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/{{payoutId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payoutId", payout_id, explode=True)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_55._unmap(response)

    @cast_models
    def cancel_payout(
        self,
        payout_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_55:
        """The payout can be canceled unless its status is confirmation or completed.
        This method triggers the Payout Canceled webhook. This webhook contains the same information as the response.

        :param payout_id: ID of the payout. String starting with **payout_**.
        :type payout_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: payout canceled successfully
        :rtype: InlineResponse200_55
        """

        Validator(str).validate(payout_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/{{payoutId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payoutId", payout_id, explode=True)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_55._unmap(response)

    @cast_models
    def simulate_payout_returned_status(
        self,
        payout: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_64:
        """Use this method in the sandbox to simulate the action of a third party that is required for returning the payout.<BR>Relevant when the status of the payout is Completed.<BR>Prerequisites:<BR>* Create Wallet<BR>* Add Funds to Wallet Account<BR>* Create Payout<BR>* Complete Payout<BR>This method triggers the 'Payout Returned' Webhook. This webhook contains the same information as the response.

        :param payout: ID of the payout. String starting with **payout_**.
        :type payout: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Displays the payout's details.
        :rtype: InlineResponse200_64
        """

        Validator(str).validate(payout)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payouts/return/{{payout}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payout", payout)
            .serialize()
            .set_method("POST")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_64._unmap(response)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class EscrowEwallets(BaseModel):
    """Describes the wallets and the releases from escrow.

    :param ewallet: ID of the wallet, a string starting with **ewallet_**., defaults to None
    :type ewallet: str, optional
    :param amount: The amount to release to this wallet. Relevant when `percentage` is not set., defaults to None
    :type amount: float, optional
    :param percentage: The percentage of this escrow to release to this wallet. Relevant when `amount` is not set. On a partial release after the first, this refers to the percentage of the original amount of the escrow., defaults to None
    :type percentage: float, optional
    """

    def __init__(
        self, ewallet: str = None, amount: float = None, percentage: float = None
    ):
        """Describes the wallets and the releases from escrow.

        :param ewallet: ID of the wallet, a string starting with **ewallet_**., defaults to None
        :type ewallet: str, optional
        :param amount: The amount to release to this wallet. Relevant when `percentage` is not set., defaults to None
        :type amount: float, optional
        :param percentage: The percentage of this escrow to release to this wallet. Relevant when `amount` is not set. On a partial release after the first, this refers to the percentage of the original amount of the escrow., defaults to None
        :type percentage: float, optional
        """
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)
        self.amount = self._define_number("amount", amount, nullable=True)
        self.percentage = self._define_number("percentage", percentage, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class InvoiceItemPeriod(BaseModel):
    """InvoiceItemPeriod

    :param start: First date in the period covered by the invoice, in Unix time. Response only., defaults to None
    :type start: str, optional
    :param end: Last date in the period covered by the invoice, in Unix time. Response only., defaults to None
    :type end: str, optional
    """

    def __init__(self, start: str = None, end: str = None):
        """InvoiceItemPeriod

        :param start: First date in the period covered by the invoice, in Unix time. Response only., defaults to None
        :type start: str, optional
        :param end: Last date in the period covered by the invoice, in Unix time. Response only., defaults to None
        :type end: str, optional
        """
        self.start = self._define_str("start", start, nullable=True)
        self.end = self._define_str("end", end, nullable=True)



from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .account import Account
from .ewallet_contacts import EwalletContacts


class EwalletCategory(Enum):
    """An enumeration representing different categories.

    :cvar COLLECT: "collect"
    :vartype COLLECT: str
    :cvar DISBURSE: "disburse"
    :vartype DISBURSE: str
    :cvar CARDAUTHORIZATION: "card_authorization"
    :vartype CARDAUTHORIZATION: str
    :cvar GENERAL: "general"
    :vartype GENERAL: str
    """

    COLLECT = "collect"
    DISBURSE = "disburse"
    CARDAUTHORIZATION = "card_authorization"
    GENERAL = "general"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, EwalletCategory._member_map_.values()))


class EwalletStatus(Enum):
    """An enumeration representing different categories.

    :cvar ACT: "ACT"
    :vartype ACT: str
    :cvar DIS: "DIS"
    :vartype DIS: str
    """

    ACT = "ACT"
    DIS = "DIS"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, EwalletStatus._member_map_.values()))


class EwalletType(Enum):
    """An enumeration representing different categories.

    :cvar COMPANY: "company"
    :vartype COMPANY: str
    :cvar PERSON: "person"
    :vartype PERSON: str
    :cvar CLIENT: "client"
    :vartype CLIENT: str
    """

    COMPANY = "company"
    PERSON = "person"
    CLIENT = "client"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, EwalletType._member_map_.values()))


@JsonMap({"id_": "id", "type_": "type"})
class Ewallet(BaseModel):
    """Ewallet

    :param accounts: accounts, defaults to None
    :type accounts: List[Account], optional
    :param category: Indicates the type of client wallet, defaults to None
    :type category: EwalletCategory, optional
    :param contacts: contacts, defaults to None
    :type contacts: EwalletContacts, optional
    :param email: Email address of the wallet owner. Response only. Deprecated., defaults to None
    :type email: str, optional
    :param ewallet_reference_id: Wallet ID defined by the customer or end user. Must be unique., defaults to None
    :type ewallet_reference_id: str, optional
    :param first_name: First name of the wallet owner., defaults to None
    :type first_name: str, optional
    :param id_: Unique identifier of the wallet object. String starting with **ewallet_**., defaults to None
    :type id_: str, optional
    :param last_name: Last name of the wallet owner., defaults to None
    :type last_name: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param phone_number: Phone number of the wallet owner in E.164 format. Response only. Deprecated., defaults to None
    :type phone_number: str, optional
    :param status: Status of the wallet, defaults to None
    :type status: EwalletStatus, optional
    :param type_: Type of wallet., defaults to None
    :type type_: EwalletType, optional
    :param verification_status: Result of the verification check., defaults to None
    :type verification_status: str, optional
    """

    def __init__(
        self,
        accounts: List[Account] = None,
        category: EwalletCategory = None,
        contacts: EwalletContacts = None,
        email: str = None,
        ewallet_reference_id: str = None,
        first_name: str = None,
        id_: str = None,
        last_name: str = None,
        metadata: dict = None,
        phone_number: str = None,
        status: EwalletStatus = None,
        type_: EwalletType = None,
        verification_status: str = None,
    ):
        """Ewallet

        :param accounts: accounts, defaults to None
        :type accounts: List[Account], optional
        :param category: Indicates the type of client wallet, defaults to None
        :type category: EwalletCategory, optional
        :param contacts: contacts, defaults to None
        :type contacts: EwalletContacts, optional
        :param email: Email address of the wallet owner. Response only. Deprecated., defaults to None
        :type email: str, optional
        :param ewallet_reference_id: Wallet ID defined by the customer or end user. Must be unique., defaults to None
        :type ewallet_reference_id: str, optional
        :param first_name: First name of the wallet owner., defaults to None
        :type first_name: str, optional
        :param id_: Unique identifier of the wallet object. String starting with **ewallet_**., defaults to None
        :type id_: str, optional
        :param last_name: Last name of the wallet owner., defaults to None
        :type last_name: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param phone_number: Phone number of the wallet owner in E.164 format. Response only. Deprecated., defaults to None
        :type phone_number: str, optional
        :param status: Status of the wallet, defaults to None
        :type status: EwalletStatus, optional
        :param type_: Type of wallet., defaults to None
        :type type_: EwalletType, optional
        :param verification_status: Result of the verification check., defaults to None
        :type verification_status: str, optional
        """
        self.accounts = self._define_list(accounts, Account)
        self.category = (
            self._enum_matching(category, EwalletCategory.list(), "category")
            if category
            else None
        )
        self.contacts = self._define_object(contacts, EwalletContacts)
        self.email = self._define_str("email", email, nullable=True)
        self.ewallet_reference_id = self._define_str(
            "ewallet_reference_id", ewallet_reference_id, nullable=True
        )
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.last_name = self._define_str("last_name", last_name, nullable=True)
        self.metadata = metadata
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )
        self.status = (
            self._enum_matching(status, EwalletStatus.list(), "status")
            if status
            else None
        )
        self.type_ = (
            self._enum_matching(type_, EwalletType.list(), "type_") if type_ else None
        )
        self.verification_status = self._define_str(
            "verification_status", verification_status, nullable=True
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .status_1 import Status1
from .simulate_card_transaction_authorization_request_eea import (
    SimulateCardTransactionAuthorizationRequestEea,
)


@JsonMap({})
class InlineResponse200_87(BaseModel):
    """InlineResponse200_87

    :param status: status, defaults to None
    :type status: Status1, optional
    :param data: data, defaults to None
    :type data: SimulateCardTransactionAuthorizationRequestEea, optional
    """

    def __init__(
        self,
        status: Status1 = None,
        data: SimulateCardTransactionAuthorizationRequestEea = None,
    ):
        """InlineResponse200_87

        :param status: status, defaults to None
        :type status: Status1, optional
        :param data: data, defaults to None
        :type data: SimulateCardTransactionAuthorizationRequestEea, optional
        """
        self.status = self._define_object(status, Status1)
        self.data = self._define_object(
            data, SimulateCardTransactionAuthorizationRequestEea
        )



from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .simulate_card_transaction_authorization_request_eea_auth_response import (
    SimulateCardTransactionAuthorizationRequestEeaAuthResponse,
)
from .simulate_clearing_card_transaction_eea_remote_auth_response import (
    SimulateClearingCardTransactionEeaRemoteAuthResponse,
)


class SimulateClearingCardTransactionEeaTxnType(Enum):
    """An enumeration representing different categories.

    :cvar ADJUSTMENT: "ADJUSTMENT"
    :vartype ADJUSTMENT: str
    :cvar ATMFEE: "ATM_FEE"
    :vartype ATMFEE: str
    :cvar CREDIT: "CREDIT"
    :vartype CREDIT: str
    :cvar REFUND: "REFUND"
    :vartype REFUND: str
    :cvar REVERSAL: "REVERSAL"
    :vartype REVERSAL: str
    :cvar SALE: "SALE"
    :vartype SALE: str
    """

    ADJUSTMENT = "ADJUSTMENT"
    ATMFEE = "ATM_FEE"
    CREDIT = "CREDIT"
    REFUND = "REFUND"
    REVERSAL = "REVERSAL"
    SALE = "SALE"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                SimulateClearingCardTransactionEeaTxnType._member_map_.values(),
            )
        )


@JsonMap({"id_": "id"})
class SimulateClearingCardTransactionEea(BaseModel):
    """SimulateClearingCardTransactionEea

    :param amount: Amount of the transaction, in units defined in `currency`., defaults to None
    :type amount: float, optional
    :param auth_response: Response related to an authorization. Contains the following fields:, defaults to None
    :type auth_response: SimulateCardTransactionAuthorizationRequestEeaAuthResponse, optional
    :param card_id: ID of the card. String starting with **card_**., defaults to None
    :type card_id: str, optional
    :param card_transaction_id: ID of the transaction. String starting with **cit_**., defaults to None
    :type card_transaction_id: str, optional
    :param currency_code: Currency of the transaction. Three-letter ISO 4217 code.Currency of the refund transaction. Three-letter ISO 4217 code., defaults to None
    :type currency_code: str, optional
    :param fx_rate: The exchange rate. Relevant to capture (clearing) transactions., defaults to None
    :type fx_rate: str, optional
    :param id_: ID of the issued card transaction. String starting with **cit_**., defaults to None
    :type id_: str, optional
    :param is_remote_auth: Indicates whether remote authorization is enabled., defaults to None
    :type is_remote_auth: bool, optional
    :param merchant_category_code: Four-digit merchant category code (MCC) of the initiator of the transaction, as defined in ISO 18245., defaults to None
    :type merchant_category_code: str, optional
    :param merchant_identification_code: Reserved., defaults to None
    :type merchant_identification_code: str, optional
    :param merchant_name_location: Name and location of the merchant. Maximum 40 characters., defaults to None
    :type merchant_name_location: str, optional
    :param remote_auth_endpoint: The URL where the remote authorization is sent., defaults to None
    :type remote_auth_endpoint: str, optional
    :param remote_auth_response: Response to a successful remote authorization request. Contains the following fields:, defaults to None
    :type remote_auth_response: SimulateClearingCardTransactionEeaRemoteAuthResponse, optional
    :param transaction_amount: Amount debited from the Rapyd Wallet. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015., defaults to None
    :type transaction_amount: float, optional
    :param transaction_currency_code: Original transaction currency, if different from the currency credited to the Rapyd Wallet. Three-letter ISO 4217 code., defaults to None
    :type transaction_currency_code: str, optional
    :param txn_type: Type of transaction., defaults to None
    :type txn_type: SimulateClearingCardTransactionEeaTxnType, optional
    """

    def __init__(
        self,
        amount: float = None,
        auth_response: SimulateCardTransactionAuthorizationRequestEeaAuthResponse = None,
        card_id: str = None,
        card_transaction_id: str = None,
        currency_code: str = None,
        fx_rate: str = None,
        id_: str = None,
        is_remote_auth: bool = None,
        merchant_category_code: str = None,
        merchant_identification_code: str = None,
        merchant_name_location: str = None,
        remote_auth_endpoint: str = None,
        remote_auth_response: SimulateClearingCardTransactionEeaRemoteAuthResponse = None,
        transaction_amount: float = None,
        transaction_currency_code: str = None,
        txn_type: SimulateClearingCardTransactionEeaTxnType = None,
    ):
        """SimulateClearingCardTransactionEea

        :param amount: Amount of the transaction, in units defined in `currency`., defaults to None
        :type amount: float, optional
        :param auth_response: Response related to an authorization. Contains the following fields:, defaults to None
        :type auth_response: SimulateCardTransactionAuthorizationRequestEeaAuthResponse, optional
        :param card_id: ID of the card. String starting with **card_**., defaults to None
        :type card_id: str, optional
        :param card_transaction_id: ID of the transaction. String starting with **cit_**., defaults to None
        :type card_transaction_id: str, optional
        :param currency_code: Currency of the transaction. Three-letter ISO 4217 code.Currency of the refund transaction. Three-letter ISO 4217 code., defaults to None
        :type currency_code: str, optional
        :param fx_rate: The exchange rate. Relevant to capture (clearing) transactions., defaults to None
        :type fx_rate: str, optional
        :param id_: ID of the issued card transaction. String starting with **cit_**., defaults to None
        :type id_: str, optional
        :param is_remote_auth: Indicates whether remote authorization is enabled., defaults to None
        :type is_remote_auth: bool, optional
        :param merchant_category_code: Four-digit merchant category code (MCC) of the initiator of the transaction, as defined in ISO 18245., defaults to None
        :type merchant_category_code: str, optional
        :param merchant_identification_code: Reserved., defaults to None
        :type merchant_identification_code: str, optional
        :param merchant_name_location: Name and location of the merchant. Maximum 40 characters., defaults to None
        :type merchant_name_location: str, optional
        :param remote_auth_endpoint: The URL where the remote authorization is sent., defaults to None
        :type remote_auth_endpoint: str, optional
        :param remote_auth_response: Response to a successful remote authorization request. Contains the following fields:, defaults to None
        :type remote_auth_response: SimulateClearingCardTransactionEeaRemoteAuthResponse, optional
        :param transaction_amount: Amount debited from the Rapyd Wallet. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015., defaults to None
        :type transaction_amount: float, optional
        :param transaction_currency_code: Original transaction currency, if different from the currency credited to the Rapyd Wallet. Three-letter ISO 4217 code., defaults to None
        :type transaction_currency_code: str, optional
        :param txn_type: Type of transaction., defaults to None
        :type txn_type: SimulateClearingCardTransactionEeaTxnType, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.auth_response = self._define_object(
            auth_response, SimulateCardTransactionAuthorizationRequestEeaAuthResponse
        )
        self.card_id = self._define_str("card_id", card_id, nullable=True)
        self.card_transaction_id = self._define_str(
            "card_transaction_id", card_transaction_id, nullable=True
        )
        self.currency_code = self._define_str(
            "currency_code", currency_code, nullable=True
        )
        self.fx_rate = self._define_str("fx_rate", fx_rate, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.is_remote_auth = is_remote_auth
        self.merchant_category_code = self._define_str(
            "merchant_category_code", merchant_category_code, nullable=True
        )
        self.merchant_identification_code = self._define_str(
            "merchant_identification_code", merchant_identification_code, nullable=True
        )
        self.merchant_name_location = self._define_str(
            "merchant_name_location", merchant_name_location, nullable=True
        )
        self.remote_auth_endpoint = self._define_str(
            "remote_auth_endpoint", remote_auth_endpoint, nullable=True
        )
        self.remote_auth_response = self._define_object(
            remote_auth_response, SimulateClearingCardTransactionEeaRemoteAuthResponse
        )
        self.transaction_amount = self._define_number(
            "transaction_amount", transaction_amount, nullable=True
        )
        self.transaction_currency_code = self._define_str(
            "transaction_currency_code", transaction_currency_code, nullable=True
        )
        self.txn_type = (
            self._enum_matching(
                txn_type, SimulateClearingCardTransactionEeaTxnType.list(), "txn_type"
            )
            if txn_type
            else None
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .invoice_item_response_period import InvoiceItemResponsePeriod
from .plan import Plan


@JsonMap({"id_": "id", "date_": "date"})
class InvoiceItemResponse(BaseModel):
    """Invoice item

    :param id_: ID of the invoice item. String starting with **ii_**., defaults to None
    :type id_: str, optional
    :param amount: The amount of the total charge or credit for this item. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015.\<BR\> This is `quantity` times `unit_amount`. A credit is indicated by a negative number., defaults to None
    :type amount: float, optional
    :param currency: Three-letter ISO 4217 code for the currency used in the `amount` field., defaults to None
    :type currency: str, optional
    :param customer: ID of the customer. String starting with **cus_**., defaults to None
    :type customer: str, optional
    :param date_: The time of the charge or credit, in Unix time., defaults to None
    :type date_: str, optional
    :param description: item description, defaults to None
    :type description: str, optional
    :param discountable: Determines whether this invoice item is subject to the discount defined in the coupon that is assigned to the customer or subscription., defaults to None
    :type discountable: bool, optional
    :param invoice_item: ID of the invoice item. String starting with **ii_**., defaults to None
    :type invoice_item: str, optional
    :param invoice: ID of the invoice that this invoice item is assigned to. Relevant when `subscription` is not set., defaults to None
    :type invoice: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param period: Defines the start and end of the time period that this invoice item refers to. Relevant when the invoice item refers to more than one day. Contains the following fields:, defaults to None
    :type period: InvoiceItemResponsePeriod, optional
    :param plan: Describes the pricing structure for the invoice item. For details of the fields in the `plan` object, see 'Create Plan'., defaults to None
    :type plan: Plan, optional
    :param proration: Indicates whether the invoice item is prorated., defaults to None
    :type proration: bool, optional
    :param quantity: Indicates the number of units charged as a single invoice item. Integer., defaults to None
    :type quantity: float, optional
    :param subscription: ID of the subscription this invoice item is assigned to. By default, the invoice item is assigned to the customer's subscription whose current billing cycle ends first. Relevant when `invoice` is not set., defaults to None
    :type subscription: str, optional
    :param unit_amount: Per-unit price of the product or service, adjusted as defined in the plan. Decimal., defaults to None
    :type unit_amount: float, optional
    """

    def __init__(
        self,
        id_: str = None,
        amount: float = None,
        currency: str = None,
        customer: str = None,
        date_: str = None,
        description: str = None,
        discountable: bool = None,
        invoice_item: str = None,
        invoice: str = None,
        metadata: dict = None,
        period: InvoiceItemResponsePeriod = None,
        plan: Plan = None,
        proration: bool = None,
        quantity: float = None,
        subscription: str = None,
        unit_amount: float = None,
    ):
        """Invoice item

        :param id_: ID of the invoice item. String starting with **ii_**., defaults to None
        :type id_: str, optional
        :param amount: The amount of the total charge or credit for this item. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015.\<BR\> This is `quantity` times `unit_amount`. A credit is indicated by a negative number., defaults to None
        :type amount: float, optional
        :param currency: Three-letter ISO 4217 code for the currency used in the `amount` field., defaults to None
        :type currency: str, optional
        :param customer: ID of the customer. String starting with **cus_**., defaults to None
        :type customer: str, optional
        :param date_: The time of the charge or credit, in Unix time., defaults to None
        :type date_: str, optional
        :param description: item description, defaults to None
        :type description: str, optional
        :param discountable: Determines whether this invoice item is subject to the discount defined in the coupon that is assigned to the customer or subscription., defaults to None
        :type discountable: bool, optional
        :param invoice_item: ID of the invoice item. String starting with **ii_**., defaults to None
        :type invoice_item: str, optional
        :param invoice: ID of the invoice that this invoice item is assigned to. Relevant when `subscription` is not set., defaults to None
        :type invoice: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param period: Defines the start and end of the time period that this invoice item refers to. Relevant when the invoice item refers to more than one day. Contains the following fields:, defaults to None
        :type period: InvoiceItemResponsePeriod, optional
        :param plan: Describes the pricing structure for the invoice item. For details of the fields in the `plan` object, see 'Create Plan'., defaults to None
        :type plan: Plan, optional
        :param proration: Indicates whether the invoice item is prorated., defaults to None
        :type proration: bool, optional
        :param quantity: Indicates the number of units charged as a single invoice item. Integer., defaults to None
        :type quantity: float, optional
        :param subscription: ID of the subscription this invoice item is assigned to. By default, the invoice item is assigned to the customer's subscription whose current billing cycle ends first. Relevant when `invoice` is not set., defaults to None
        :type subscription: str, optional
        :param unit_amount: Per-unit price of the product or service, adjusted as defined in the plan. Decimal., defaults to None
        :type unit_amount: float, optional
        """
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.amount = self._define_number("amount", amount, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.customer = self._define_str("customer", customer, nullable=True)
        self.date_ = self._define_str("date_", date_, nullable=True)
        self.description = self._define_str("description", description, nullable=True)
        self.discountable = discountable
        self.invoice_item = self._define_str(
            "invoice_item", invoice_item, nullable=True
        )
        self.invoice = self._define_str("invoice", invoice, nullable=True)
        self.metadata = metadata
        self.period = self._define_object(period, InvoiceItemResponsePeriod)
        self.plan = self._define_object(plan, Plan)
        self.proration = proration
        self.quantity = self._define_number("quantity", quantity, nullable=True)
        self.subscription = self._define_str(
            "subscription", subscription, nullable=True
        )
        self.unit_amount = self._define_number(
            "unit_amount", unit_amount, nullable=True
        )



from __future__ import annotations
from enum import Enum
from typing import Union
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel
from .address_1 import Address1
from .client_details_object import ClientDetailsObject
from .fee import Fee
from .payment_method_type import PaymentMethodType


class InitiationType(Enum):
    """An enumeration representing different categories.

    :cvar CUSTOMERPRESENT: "customer_present"
    :vartype CUSTOMERPRESENT: str
    :cvar INSTALLMENT: "installment"
    :vartype INSTALLMENT: str
    :cvar MOTO: "moto"
    :vartype MOTO: str
    :cvar RECURRING: "recurring"
    :vartype RECURRING: str
    :cvar UNSCHEDULED: "unscheduled"
    :vartype UNSCHEDULED: str
    """

    CUSTOMERPRESENT = "customer_present"
    INSTALLMENT = "installment"
    MOTO = "moto"
    RECURRING = "recurring"
    UNSCHEDULED = "unscheduled"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, InitiationType._member_map_.values()))


class V1PaymentsBodyPaymentMethodGuard(OneOfBaseModel):
    class_list = {"str": str, "PaymentMethodType": PaymentMethodType}


V1PaymentsBodyPaymentMethod = Union[str, PaymentMethodType]


@JsonMap({})
class V1PaymentsBody(BaseModel):
    """V1PaymentsBody

    :param address: address associated with this specific Rapyd entity Payment/Customer etc., defaults to None
    :type address: Address1, optional
    :param amount: The amount received by the recipient, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. To verify a card, set to 0.
    :type amount: float
    :param capture: Determines when the payment is processed for capture. When true, the payment is captured immediately. When false, the payment is captured at a later time. Relevant to cards. Default is true., defaults to None
    :type capture: bool, optional
    :param client_details: Describes the fields in the client_details object in REST messages for payments. The client_details object describes the browser that the customer is using. The client collects this information and sends it as part of the Create Payment request. This information is used for processing the 3DS version 2 authentication of the customer. Note that Client Details information is not returned in the API response and it does not appear in any webhooks., defaults to None
    :type client_details: ClientDetailsObject, optional
    :param complete_payment_url: URL where the customer is redirected after successfully completing an operation on an external page., defaults to None
    :type complete_payment_url: str, optional
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer.
    :type currency: str
    :param customer: string - ID of the customer who is making the payment. String starting with cus_. Required if payment_method is blank. object - Contains all required fields, defaults to None
    :type customer: str, optional
    :param description: Description of the payment, defaults to None
    :type description: str, optional
    :param error_payment_url: URL where the customer is redirected in case of an error in the operation. Provided by the client.Relevant to bank redirect payment methods, defaults to None
    :type error_payment_url: str, optional
    :param escrow: Determines whether the payment is held in escrow for later release., defaults to None
    :type escrow: bool, optional
    :param escrow_release_days: Determines the number of days after creation of the payment that funds are released from escrow. Funds are released at 5:00 pm GMT on the day indicated. Integer, range between 1-90. Default is 90., defaults to None
    :type escrow_release_days: int, optional
    :param ewallet: ID of the wallet that the money is paid into. String starting with **ewallet_**. Relevant when the request includes a single wallet., defaults to None
    :type ewallet: str, optional
    :param ewallets: Represents the wallets that the money is paid into. Array of objects., defaults to None
    :type ewallets: any, optional
    :param expiration: End of the time allowed for customer to make this payment, in Unix time. Must be after the current time, defaults to None
    :type expiration: float, optional
    :param fixed_side: Indicates whether the FX rate is fixed for the buy side (seller's Rapyd Wallet) or for the sell side (buyer)., defaults to None
    :type fixed_side: str, optional
    :param group_payment: ID of the group payment, a string starting with 'gp_'. Relevant to replacing a payment that failed., defaults to None
    :type group_payment: str, optional
    :param initiation_type: Describes how the transaction was initiated., defaults to None
    :type initiation_type: InitiationType, optional
    :param merchant_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param payment_fees: payment_fees, defaults to None
    :type payment_fees: Fee, optional
    :param payment_method: Payment Method object or ID., defaults to None
    :type payment_method: V1PaymentsBodyPaymentMethod, optional
    :param payment_method_options: Object describing additional information required for the payment. The object is returned by Get Payment Method Required Fields. Note that Transactions with 3DS authentication must be authenticated within 15 minutes, after which the possibility to authenticate expires., defaults to None
    :type payment_method_options: dict, optional
    :param receipt_email: Email address that the receipt for this transaction is sent to., defaults to None
    :type receipt_email: str, optional
    :param requested_currency: When fixed_side is sell, it is the currency received in the Rapyd Wallet. . When fixed_side is buy, it is the currency charged to the buyer (customer). Three-letter ISO 4217 code., defaults to None
    :type requested_currency: str, optional
    :param statement_descriptor: A text description that appears in the customer's bank statement., defaults to None
    :type statement_descriptor: str, optional
    """

    def __init__(
        self,
        amount: float,
        currency: str,
        address: Address1 = None,
        capture: bool = None,
        client_details: ClientDetailsObject = None,
        complete_payment_url: str = None,
        customer: str = None,
        description: str = None,
        error_payment_url: str = None,
        escrow: bool = None,
        escrow_release_days: int = None,
        ewallet: str = None,
        ewallets: any = None,
        expiration: float = None,
        fixed_side: str = None,
        group_payment: str = None,
        initiation_type: InitiationType = None,
        merchant_reference_id: str = None,
        metadata: dict = None,
        payment_fees: Fee = None,
        payment_method: V1PaymentsBodyPaymentMethod = None,
        payment_method_options: dict = None,
        receipt_email: str = None,
        requested_currency: str = None,
        statement_descriptor: str = None,
    ):
        """V1PaymentsBody

        :param address: address associated with this specific Rapyd entity Payment/Customer etc., defaults to None
        :type address: Address1, optional
        :param amount: The amount received by the recipient, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. To verify a card, set to 0.
        :type amount: float
        :param capture: Determines when the payment is processed for capture. When true, the payment is captured immediately. When false, the payment is captured at a later time. Relevant to cards. Default is true., defaults to None
        :type capture: bool, optional
        :param client_details: Describes the fields in the client_details object in REST messages for payments. The client_details object describes the browser that the customer is using. The client collects this information and sends it as part of the Create Payment request. This information is used for processing the 3DS version 2 authentication of the customer. Note that Client Details information is not returned in the API response and it does not appear in any webhooks., defaults to None
        :type client_details: ClientDetailsObject, optional
        :param complete_payment_url: URL where the customer is redirected after successfully completing an operation on an external page., defaults to None
        :type complete_payment_url: str, optional
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer.
        :type currency: str
        :param customer: string - ID of the customer who is making the payment. String starting with cus_. Required if payment_method is blank. object - Contains all required fields, defaults to None
        :type customer: str, optional
        :param description: Description of the payment, defaults to None
        :type description: str, optional
        :param error_payment_url: URL where the customer is redirected in case of an error in the operation. Provided by the client.Relevant to bank redirect payment methods, defaults to None
        :type error_payment_url: str, optional
        :param escrow: Determines whether the payment is held in escrow for later release., defaults to None
        :type escrow: bool, optional
        :param escrow_release_days: Determines the number of days after creation of the payment that funds are released from escrow. Funds are released at 5:00 pm GMT on the day indicated. Integer, range between 1-90. Default is 90., defaults to None
        :type escrow_release_days: int, optional
        :param ewallet: ID of the wallet that the money is paid into. String starting with **ewallet_**. Relevant when the request includes a single wallet., defaults to None
        :type ewallet: str, optional
        :param ewallets: Represents the wallets that the money is paid into. Array of objects., defaults to None
        :type ewallets: any, optional
        :param expiration: End of the time allowed for customer to make this payment, in Unix time. Must be after the current time, defaults to None
        :type expiration: float, optional
        :param fixed_side: Indicates whether the FX rate is fixed for the buy side (seller's Rapyd Wallet) or for the sell side (buyer)., defaults to None
        :type fixed_side: str, optional
        :param group_payment: ID of the group payment, a string starting with 'gp_'. Relevant to replacing a payment that failed., defaults to None
        :type group_payment: str, optional
        :param initiation_type: Describes how the transaction was initiated., defaults to None
        :type initiation_type: InitiationType, optional
        :param merchant_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param payment_fees: payment_fees, defaults to None
        :type payment_fees: Fee, optional
        :param payment_method: Payment Method object or ID., defaults to None
        :type payment_method: V1PaymentsBodyPaymentMethod, optional
        :param payment_method_options: Object describing additional information required for the payment. The object is returned by Get Payment Method Required Fields. Note that Transactions with 3DS authentication must be authenticated within 15 minutes, after which the possibility to authenticate expires., defaults to None
        :type payment_method_options: dict, optional
        :param receipt_email: Email address that the receipt for this transaction is sent to., defaults to None
        :type receipt_email: str, optional
        :param requested_currency: When fixed_side is sell, it is the currency received in the Rapyd Wallet. . When fixed_side is buy, it is the currency charged to the buyer (customer). Three-letter ISO 4217 code., defaults to None
        :type requested_currency: str, optional
        :param statement_descriptor: A text description that appears in the customer's bank statement., defaults to None
        :type statement_descriptor: str, optional
        """
        self.address = self._define_object(address, Address1)
        self.amount = amount
        self.capture = capture
        self.client_details = self._define_object(client_details, ClientDetailsObject)
        self.complete_payment_url = self._define_str(
            "complete_payment_url", complete_payment_url, nullable=True
        )
        self.currency = self._define_str(
            "currency",
            currency,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.customer = self._define_str("customer", customer, nullable=True)
        self.description = self._define_str("description", description, nullable=True)
        self.error_payment_url = self._define_str(
            "error_payment_url", error_payment_url, nullable=True
        )
        self.escrow = escrow
        self.escrow_release_days = self._define_number(
            "escrow_release_days", escrow_release_days, nullable=True
        )
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)
        self.ewallets = ewallets
        self.expiration = self._define_number("expiration", expiration, nullable=True)
        self.fixed_side = self._define_str("fixed_side", fixed_side, nullable=True)
        self.group_payment = self._define_str(
            "group_payment", group_payment, nullable=True
        )
        self.initiation_type = (
            self._enum_matching(
                initiation_type, InitiationType.list(), "initiation_type"
            )
            if initiation_type
            else None
        )
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.metadata = metadata
        self.payment_fees = self._define_object(payment_fees, Fee)
        self.payment_method = V1PaymentsBodyPaymentMethodGuard.return_one_of(
            payment_method
        )
        self.payment_method_options = payment_method_options
        self.receipt_email = self._define_str(
            "receipt_email", receipt_email, nullable=True
        )
        self.requested_currency = self._define_str(
            "requested_currency", requested_currency, nullable=True
        )
        self.statement_descriptor = self._define_str(
            "statement_descriptor", statement_descriptor, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class Status(BaseModel):
    """Status

    :param error_code: Error code of the API request. Empty when response is successful., defaults to None
    :type error_code: str, optional
    :param message: Description of the API error message. Empty when response is successful., defaults to None
    :type message: str, optional
    :param operation_id: Unique identifier (UUID)) to track the request., defaults to None
    :type operation_id: str, optional
    :param response_code: Response code of the API error message. Empty when response is successful., defaults to None
    :type response_code: str, optional
    :param status: Status of the API indicating success or failure., defaults to None
    :type status: bool, optional
    """

    def __init__(
        self,
        error_code: str = None,
        message: str = None,
        operation_id: str = None,
        response_code: str = None,
        status: bool = None,
    ):
        """Status

        :param error_code: Error code of the API request. Empty when response is successful., defaults to None
        :type error_code: str, optional
        :param message: Description of the API error message. Empty when response is successful., defaults to None
        :type message: str, optional
        :param operation_id: Unique identifier (UUID)) to track the request., defaults to None
        :type operation_id: str, optional
        :param response_code: Response code of the API error message. Empty when response is successful., defaults to None
        :type response_code: str, optional
        :param status: Status of the API indicating success or failure., defaults to None
        :type status: bool, optional
        """
        self.error_code = self._define_str("error_code", error_code, nullable=True)
        self.message = self._define_str("message", message, nullable=True)
        self.operation_id = self._define_str(
            "operation_id", operation_id, nullable=True
        )
        self.response_code = self._define_str(
            "response_code", response_code, nullable=True
        )
        self.status = status



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .inline_response_200_91_data import InlineResponse200_91Data
from .status_1 import Status1


@JsonMap({})
class InlineResponse200_91(BaseModel):
    """InlineResponse200_91

    :param data: data, defaults to None
    :type data: InlineResponse200_91Data, optional
    :param status: status, defaults to None
    :type status: Status1, optional
    """

    def __init__(self, data: InlineResponse200_91Data = None, status: Status1 = None):
        """InlineResponse200_91

        :param data: data, defaults to None
        :type data: InlineResponse200_91Data, optional
        :param status: status, defaults to None
        :type status: Status1, optional
        """
        self.data = self._define_object(data, InlineResponse200_91Data)
        self.status = self._define_object(status, Status1)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class SimulateCardTransactionAuthorizationRequestEeaAuthResponse(BaseModel):
    """Response related to an authorization. Contains the following fields:

    :param code: Indicates success or reason for failure., defaults to None
    :type code: str, optional
    :param message: Descriptive text., defaults to None
    :type message: str, optional
    """

    def __init__(self, code: str = None, message: str = None):
        """Response related to an authorization. Contains the following fields:

        :param code: Indicates success or reason for failure., defaults to None
        :type code: str, optional
        :param message: Descriptive text., defaults to None
        :type message: str, optional
        """
        self.code = self._define_str("code", code, nullable=True)
        self.message = self._define_str("message", message, nullable=True)



from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .inline_response_200_119_data_match_stats import (
    InlineResponse200_119DataMatchStats,
)
from .inline_response_200_119_data_matches import InlineResponse200_119DataMatches
from .inline_response_200_119_data_query_info import InlineResponse200_119DataQueryInfo
from .v1cnltermination_query_search_criteria import V1cnlterminationQuerySearchCriteria
from .inline_response_200_119_data_principals import InlineResponse200_119DataPrincipals
from .inline_response_200_119_data_matched_merchant import (
    InlineResponse200_119DataMatchedMerchant,
)
from .inline_response_200_119_data_registration_info import (
    InlineResponse200_119DataRegistrationInfo,
)


class InlineResponse200_119DataStatus(Enum):
    """An enumeration representing different categories.

    :cvar INPROGRESS: "IN_PROGRESS"
    :vartype INPROGRESS: str
    :cvar ERROR: "ERROR"
    :vartype ERROR: str
    :cvar INPROGRESS1: "IN_PROGRESS"
    :vartype INPROGRESS1: str
    :cvar PARTIAL: "PARTIAL"
    :vartype PARTIAL: str
    :cvar PROCESSED: "PROCESSED"
    :vartype PROCESSED: str
    """

    INPROGRESS = "IN_PROGRESS"
    ERROR = "ERROR"
    INPROGRESS1 = "IN_PROGRESS"
    PARTIAL = "PARTIAL"
    PROCESSED = "PROCESSED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value, InlineResponse200_119DataStatus._member_map_.values()
            )
        )


@JsonMap({})
class InlineResponse200_119Data(BaseModel):
    """InlineResponse200_119Data

    :param match_stats: Statistics about the query., defaults to None
    :type match_stats: InlineResponse200_119DataMatchStats, optional
    :param matches: Describes the results of the query., defaults to None
    :type matches: InlineResponse200_119DataMatches, optional
    :param query_info: asd, defaults to None
    :type query_info: InlineResponse200_119DataQueryInfo, optional
    :param search_criteria: Specifies search criteria for the query., defaults to None
    :type search_criteria: V1cnlterminationQuerySearchCriteria, optional
    :param principals: Details of the registered principal owners of the merchant.\<BR\> Maximum - 3., defaults to None
    :type principals: InlineResponse200_119DataPrincipals, optional
    :param status: Status of the Card Network Lookup Service query. One of the following:\<BR\>* **ERROR** - The query failed.\<BR\>* **IN_PROGRESS** - The query is being processed. Results are not available.\<BR\>* **PARTIAL** - The query process is partially complete. Some results are available.\<BR\>* **PROCESSED** - The query process is complete. All results are available.The two-letter ISO 3166-1 ALPHA-2 code for the country of the identification document., defaults to None
    :type status: InlineResponse200_119DataStatus, optional
    :param matched_merchant: Data about the merchant on file in the database., defaults to None
    :type matched_merchant: InlineResponse200_119DataMatchedMerchant, optional
    :param registration_info: Registration information in the card network's termination database about the merchant and the acquirer that listed the merchant., defaults to None
    :type registration_info: InlineResponse200_119DataRegistrationInfo, optional
    """

    def __init__(
        self,
        match_stats: InlineResponse200_119DataMatchStats = None,
        matches: InlineResponse200_119DataMatches = None,
        query_info: InlineResponse200_119DataQueryInfo = None,
        search_criteria: V1cnlterminationQuerySearchCriteria = None,
        principals: InlineResponse200_119DataPrincipals = None,
        status: InlineResponse200_119DataStatus = None,
        matched_merchant: InlineResponse200_119DataMatchedMerchant = None,
        registration_info: InlineResponse200_119DataRegistrationInfo = None,
    ):
        """InlineResponse200_119Data

        :param match_stats: Statistics about the query., defaults to None
        :type match_stats: InlineResponse200_119DataMatchStats, optional
        :param matches: Describes the results of the query., defaults to None
        :type matches: InlineResponse200_119DataMatches, optional
        :param query_info: asd, defaults to None
        :type query_info: InlineResponse200_119DataQueryInfo, optional
        :param search_criteria: Specifies search criteria for the query., defaults to None
        :type search_criteria: V1cnlterminationQuerySearchCriteria, optional
        :param principals: Details of the registered principal owners of the merchant.\<BR\> Maximum - 3., defaults to None
        :type principals: InlineResponse200_119DataPrincipals, optional
        :param status: Status of the Card Network Lookup Service query. One of the following:\<BR\>* **ERROR** - The query failed.\<BR\>* **IN_PROGRESS** - The query is being processed. Results are not available.\<BR\>* **PARTIAL** - The query process is partially complete. Some results are available.\<BR\>* **PROCESSED** - The query process is complete. All results are available.The two-letter ISO 3166-1 ALPHA-2 code for the country of the identification document., defaults to None
        :type status: InlineResponse200_119DataStatus, optional
        :param matched_merchant: Data about the merchant on file in the database., defaults to None
        :type matched_merchant: InlineResponse200_119DataMatchedMerchant, optional
        :param registration_info: Registration information in the card network's termination database about the merchant and the acquirer that listed the merchant., defaults to None
        :type registration_info: InlineResponse200_119DataRegistrationInfo, optional
        """
        self.match_stats = self._define_object(
            match_stats, InlineResponse200_119DataMatchStats
        )
        self.matches = self._define_object(matches, InlineResponse200_119DataMatches)
        self.query_info = self._define_object(
            query_info, InlineResponse200_119DataQueryInfo
        )
        self.search_criteria = self._define_object(
            search_criteria, V1cnlterminationQuerySearchCriteria
        )
        self.principals = self._define_object(
            principals, InlineResponse200_119DataPrincipals
        )
        self.status = (
            self._enum_matching(
                status, InlineResponse200_119DataStatus.list(), "status"
            )
            if status
            else None
        )
        self.matched_merchant = self._define_object(
            matched_merchant, InlineResponse200_119DataMatchedMerchant
        )
        self.registration_info = self._define_object(
            registration_info, InlineResponse200_119DataRegistrationInfo
        )



from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .address_1 import Address1
from .payment_amount_range_per_currency_inner import PaymentAmountRangePerCurrencyInner
from .bin_details import BinDetails
from .category import Category
from .condition import Condition
from .customer import Customer


class PaymentOptionsPaymentFlowType(Enum):
    """An enumeration representing different categories.

    :cvar DIRECT: "direct"
    :vartype DIRECT: str
    :cvar EWALLET: "eWallet"
    :vartype EWALLET: str
    :cvar EWALLETPAYOUT: "ewallet_payout"
    :vartype EWALLETPAYOUT: str
    :cvar CARD: "card"
    :vartype CARD: str
    :cvar REDIRECTURL: "redirect_url"
    :vartype REDIRECTURL: str
    """

    DIRECT = "direct"
    EWALLET = "eWallet"
    EWALLETPAYOUT = "ewallet_payout"
    CARD = "card"
    REDIRECTURL = "redirect_url"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, PaymentOptionsPaymentFlowType._member_map_.values())
        )


@JsonMap({})
class PaymentOptions(BaseModel):
    """A payment method type is a type of payment method that any customer can use, for example, ee_mastercard_card, Mastercard for Estonia. When it is added to a customer profile, it becomes a payment method that is specific to that one customer. The name of the payment method type starts with a prefix for the country, the 2-letter ISO 3166-1 alpha-2 code. If the payment method is valid in multiple countries, the prefix is xx_. The payment method type has a suffix with one of the following values - _bank - Bank transfer or bank redirect _card - Credit card, debit card or other card _cash - Cash _ewallet - Local eWallet

    :param address: address associated with this specific Rapyd entity Payment/Customer etc., defaults to None
    :type address: Address1, optional
    :param amount_range_per_currency: Indicates the amount range for the payment method's currencies. Each object contains the following fields:* currency - Three-letter ISO 4217 format of currency* maximum_amount - The maximum payment amount* minimum_amount - The minimum payment amount, defaults to None
    :type amount_range_per_currency: List[PaymentAmountRangePerCurrencyInner], optional
    :param bin_details: Bank Identification Number (BIN) details. Read-only. Object containing the following fields - * bin_number - BIN number * country - The two-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase. * funding - Type of card funding. One of the following [credit, debit, prepaid, unknown] * bank - Name of the issuing bank. Relevant to cards, defaults to None
    :type bin_details: BinDetails, optional
    :param category: category, defaults to None
    :type category: Category, optional
    :param conditions: conditions, defaults to None
    :type conditions: List[Condition], optional
    :param country: country, defaults to None
    :type country: str, optional
    :param currencies: currencies, defaults to None
    :type currencies: List[str], optional
    :param customer: customer, defaults to None
    :type customer: Customer, optional
    :param fingerprint: Hash of the card number, expiration date and CVV. Read-only. Relevant to cards, defaults to None
    :type fingerprint: str, optional
    :param image: A URL to the image of the icon for the type of payment method. Response only, defaults to None
    :type image: str, optional
    :param instructions: Instructions for the field. Response only, defaults to None
    :type instructions: str, optional
    :param is_cancelable: Indicates whether a payment made with this payment method can be canceled. Response only, defaults to None
    :type is_cancelable: bool, optional
    :param is_expirable: Indicates whether the merchant can set an expiration time for the customer to complete the payment. Response only, defaults to None
    :type is_expirable: bool, optional
    :param is_online: Indicates whether the payment is completed immediately online. Response only, defaults to None
    :type is_online: bool, optional
    :param is_refundable: Indicates whether the payment method type supports refunds, defaults to None
    :type is_refundable: bool, optional
    :param is_required: Whether the field is required for using the payment method. When the conditions defined by conditions are met, the field is required even though the value of is_required is false. Response only, defaults to None
    :type is_required: bool, optional
    :param is_tokenizable: Indicates whether the token of the payment method can be used in a collect operation, defaults to None
    :type is_tokenizable: bool, optional
    :param is_virtual: Indicates whether a Web-based version of the payment method type exists, defaults to None
    :type is_virtual: bool, optional
    :param last4: last4 - Last four digits of the card number. Read-only. Relevant to cards, defaults to None
    :type last4: str, optional
    :param maximum_expiration_seconds: The maximum time (in seconds) that the merchant can set for completing the payment. Relevant when is_expirable is true. Response only, defaults to None
    :type maximum_expiration_seconds: int, optional
    :param minimum_expiration_seconds: The minimum time (in seconds) that the merchant can set for completing the payment. Relevant when is_expirable is true. Response only, defaults to None
    :type minimum_expiration_seconds: int, optional
    :param multiple_overage_allowed: Indicates whether multiple overage charges are allowed for this payment method type, defaults to None
    :type multiple_overage_allowed: bool, optional
    :param name: The name of the payment method, in user-friendly terms. For example, 'Ireland Visa card'. Response only, defaults to None
    :type name: str, optional
    :param payment_flow_type: payment_flow_type, defaults to None
    :type payment_flow_type: PaymentOptionsPaymentFlowType, optional
    """

    def __init__(
        self,
        address: Address1 = None,
        amount_range_per_currency: List[PaymentAmountRangePerCurrencyInner] = None,
        bin_details: BinDetails = None,
        category: Category = None,
        conditions: List[Condition] = None,
        country: str = None,
        currencies: List[str] = None,
        customer: Customer = None,
        fingerprint: str = None,
        image: str = None,
        instructions: str = None,
        is_cancelable: bool = None,
        is_expirable: bool = None,
        is_online: bool = None,
        is_refundable: bool = None,
        is_required: bool = None,
        is_tokenizable: bool = None,
        is_virtual: bool = None,
        last4: str = None,
        maximum_expiration_seconds: int = None,
        minimum_expiration_seconds: int = None,
        multiple_overage_allowed: bool = None,
        name: str = None,
        payment_flow_type: PaymentOptionsPaymentFlowType = None,
    ):
        """A payment method type is a type of payment method that any customer can use, for example, ee_mastercard_card, Mastercard for Estonia. When it is added to a customer profile, it becomes a payment method that is specific to that one customer. The name of the payment method type starts with a prefix for the country, the 2-letter ISO 3166-1 alpha-2 code. If the payment method is valid in multiple countries, the prefix is xx_. The payment method type has a suffix with one of the following values - _bank - Bank transfer or bank redirect _card - Credit card, debit card or other card _cash - Cash _ewallet - Local eWallet

        :param address: address associated with this specific Rapyd entity Payment/Customer etc., defaults to None
        :type address: Address1, optional
        :param amount_range_per_currency: Indicates the amount range for the payment method's currencies. Each object contains the following fields:* currency - Three-letter ISO 4217 format of currency* maximum_amount - The maximum payment amount* minimum_amount - The minimum payment amount, defaults to None
        :type amount_range_per_currency: List[PaymentAmountRangePerCurrencyInner], optional
        :param bin_details: Bank Identification Number (BIN) details. Read-only. Object containing the following fields - * bin_number - BIN number * country - The two-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase. * funding - Type of card funding. One of the following [credit, debit, prepaid, unknown] * bank - Name of the issuing bank. Relevant to cards, defaults to None
        :type bin_details: BinDetails, optional
        :param category: category, defaults to None
        :type category: Category, optional
        :param conditions: conditions, defaults to None
        :type conditions: List[Condition], optional
        :param country: country, defaults to None
        :type country: str, optional
        :param currencies: currencies, defaults to None
        :type currencies: List[str], optional
        :param customer: customer, defaults to None
        :type customer: Customer, optional
        :param fingerprint: Hash of the card number, expiration date and CVV. Read-only. Relevant to cards, defaults to None
        :type fingerprint: str, optional
        :param image: A URL to the image of the icon for the type of payment method. Response only, defaults to None
        :type image: str, optional
        :param instructions: Instructions for the field. Response only, defaults to None
        :type instructions: str, optional
        :param is_cancelable: Indicates whether a payment made with this payment method can be canceled. Response only, defaults to None
        :type is_cancelable: bool, optional
        :param is_expirable: Indicates whether the merchant can set an expiration time for the customer to complete the payment. Response only, defaults to None
        :type is_expirable: bool, optional
        :param is_online: Indicates whether the payment is completed immediately online. Response only, defaults to None
        :type is_online: bool, optional
        :param is_refundable: Indicates whether the payment method type supports refunds, defaults to None
        :type is_refundable: bool, optional
        :param is_required: Whether the field is required for using the payment method. When the conditions defined by conditions are met, the field is required even though the value of is_required is false. Response only, defaults to None
        :type is_required: bool, optional
        :param is_tokenizable: Indicates whether the token of the payment method can be used in a collect operation, defaults to None
        :type is_tokenizable: bool, optional
        :param is_virtual: Indicates whether a Web-based version of the payment method type exists, defaults to None
        :type is_virtual: bool, optional
        :param last4: last4 - Last four digits of the card number. Read-only. Relevant to cards, defaults to None
        :type last4: str, optional
        :param maximum_expiration_seconds: The maximum time (in seconds) that the merchant can set for completing the payment. Relevant when is_expirable is true. Response only, defaults to None
        :type maximum_expiration_seconds: int, optional
        :param minimum_expiration_seconds: The minimum time (in seconds) that the merchant can set for completing the payment. Relevant when is_expirable is true. Response only, defaults to None
        :type minimum_expiration_seconds: int, optional
        :param multiple_overage_allowed: Indicates whether multiple overage charges are allowed for this payment method type, defaults to None
        :type multiple_overage_allowed: bool, optional
        :param name: The name of the payment method, in user-friendly terms. For example, 'Ireland Visa card'. Response only, defaults to None
        :type name: str, optional
        :param payment_flow_type: payment_flow_type, defaults to None
        :type payment_flow_type: PaymentOptionsPaymentFlowType, optional
        """
        self.address = self._define_object(address, Address1)
        self.amount_range_per_currency = self._define_list(
            amount_range_per_currency, PaymentAmountRangePerCurrencyInner
        )
        self.bin_details = self._define_object(bin_details, BinDetails)
        self.category = (
            self._enum_matching(category, Category.list(), "category")
            if category
            else None
        )
        self.conditions = self._define_list(conditions, Condition)
        self.country = self._define_str(
            "country",
            country,
            nullable=True,
            pattern="Name of the country. Two-letter ISO 3166-1 alpha-2 code.",
        )
        self.currencies = currencies
        self.customer = self._define_object(customer, Customer)
        self.fingerprint = self._define_str("fingerprint", fingerprint, nullable=True)
        self.image = self._define_str("image", image, nullable=True)
        self.instructions = self._define_str(
            "instructions", instructions, nullable=True
        )
        self.is_cancelable = is_cancelable
        self.is_expirable = is_expirable
        self.is_online = is_online
        self.is_refundable = is_refundable
        self.is_required = is_required
        self.is_tokenizable = is_tokenizable
        self.is_virtual = is_virtual
        self.last4 = self._define_str("last4", last4, nullable=True, pattern="^\d{4}$")
        self.maximum_expiration_seconds = self._define_number(
            "maximum_expiration_seconds", maximum_expiration_seconds, nullable=True
        )
        self.minimum_expiration_seconds = self._define_number(
            "minimum_expiration_seconds", minimum_expiration_seconds, nullable=True
        )
        self.multiple_overage_allowed = multiple_overage_allowed
        self.name = self._define_str("name", name, nullable=True)
        self.payment_flow_type = (
            self._enum_matching(
                payment_flow_type,
                PaymentOptionsPaymentFlowType.list(),
                "payment_flow_type",
            )
            if payment_flow_type
            else None
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .add_cardto_google_pay_response_user_address import (
    AddCardtoGooglePayResponseUserAddress,
)


@JsonMap(
    {"payment_instrument_data": "PaymentInstrumentData", "user_address": "UserAddress"}
)
class AddCardtoGooglePayResponse(BaseModel):
    """AddCardtoGooglePayResponse

    :param payment_instrument_data: The base64 encoded string that contains the encrypted object for Push Provisioning. Required for adding the card to the Google Pay wallet by using the Google Pay `pushTokenizeRequest` method. For more information refer to the Google Pay Provisioning developer documentation .Details of an account funding transaction (AFT), which transfers funds from a card to a cardholder's wallet., defaults to None
    :type payment_instrument_data: str, optional
    :param user_address: The base64 encoded string that contains the encrypted object for Push Provisioning. Required for adding the card to the Google Pay wallet by using the Google Pay `pushTokenizeRequest` method. For more information refer to the Google Pay Provisioning developer documentation .Details of an account funding transaction (AFT), which transfers funds from a card to a cardholder's wallet., defaults to None
    :type user_address: AddCardtoGooglePayResponseUserAddress, optional
    """

    def __init__(
        self,
        payment_instrument_data: str = None,
        user_address: AddCardtoGooglePayResponseUserAddress = None,
    ):
        """AddCardtoGooglePayResponse

        :param payment_instrument_data: The base64 encoded string that contains the encrypted object for Push Provisioning. Required for adding the card to the Google Pay wallet by using the Google Pay `pushTokenizeRequest` method. For more information refer to the Google Pay Provisioning developer documentation .Details of an account funding transaction (AFT), which transfers funds from a card to a cardholder's wallet., defaults to None
        :type payment_instrument_data: str, optional
        :param user_address: The base64 encoded string that contains the encrypted object for Push Provisioning. Required for adding the card to the Google Pay wallet by using the Google Pay `pushTokenizeRequest` method. For more information refer to the Google Pay Provisioning developer documentation .Details of an account funding transaction (AFT), which transfers funds from a card to a cardholder's wallet., defaults to None
        :type user_address: AddCardtoGooglePayResponseUserAddress, optional
        """
        self.payment_instrument_data = self._define_str(
            "payment_instrument_data", payment_instrument_data, nullable=True
        )
        self.user_address = self._define_object(
            user_address, AddCardtoGooglePayResponseUserAddress
        )



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    InlineResponse200_10,
    InlineResponse200_11,
    InlineResponse200_12,
    PlansPlanIdBody,
    V1PlansBody,
)


class SubscriptionPlanService(BaseService):

    @cast_models
    def list_plans(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        ending_before: str = None,
        limit: str = None,
        starting_after: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_10:
        """Retrieve a list of all plans. Use the optional query parameters to filter the results. You can filter the results further by specifying one or more Plan fields as additional query parameters.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param ending_before: The ID of the plan created after the last plan you want to retrieve., defaults to None
        :type ending_before: str, optional
        :param limit: The maximum number of plans to return. Range 1-100. Default is 10., defaults to None
        :type limit: str, optional
        :param starting_after: The ID of the plan created before the first plan you want to retrieve., defaults to None
        :type starting_after: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List of plans
        :rtype: InlineResponse200_10
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/plans", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("ending_before", ending_before, explode=False)
            .add_query("limit", limit, explode=False)
            .add_query("starting_after", starting_after, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_10._unmap(response)

    @cast_models
    def create_plan(
        self,
        request_body: V1PlansBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_11:
        """Create a pricing plan for services.

        :param request_body: The request body.
        :type request_body: V1PlansBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The created plan
        :rtype: InlineResponse200_11
        """

        Validator(V1PlansBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/plans", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_11._unmap(response)

    @cast_models
    def retrieve_plan(
        self,
        plan_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_11:
        """Retrieve the details of a pricing plan for services.

        :param plan_id: ID of the plan.
        :type plan_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The Plan
        :rtype: InlineResponse200_11
        """

        Validator(str).validate(plan_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/plans/{{planId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("planId", plan_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_11._unmap(response)

    @cast_models
    def update_plan(
        self,
        request_body: PlansPlanIdBody,
        plan_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_11:
        """Change or modify a pricing plan for services. You can update a plan's nickname or metadata.

        :param request_body: The request body.
        :type request_body: PlansPlanIdBody
        :param plan_id: ID of the plan.
        :type plan_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The Plan after updated
        :rtype: InlineResponse200_11
        """

        Validator(PlansPlanIdBody).validate(request_body)
        Validator(str).validate(plan_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/plans/{{planId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("planId", plan_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_11._unmap(response)

    @cast_models
    def delete_plan(
        self,
        plan_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_12:
        """Delete a pricing plan for services.

        :param plan_id: ID of the plan.
        :type plan_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Rapyd response with operation result, and plan id
        :rtype: InlineResponse200_12
        """

        Validator(str).validate(plan_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/plans/{{planId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("planId", plan_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_12._unmap(response)
